#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <Imlib2.h>

/* main program */
int main(int argc, char **argv)
{
  /* an image handle */
  Imlib_Image WhiteBackground;
  Imlib_Image BlackBackgnd;
  Imlib_Image Dest;
  
  /* if we provided < 3 arguments after the command - exit */
  if (argc != 4)
    {
      fprintf(stderr,"Compares two version of the same image:\n");
      fprintf(stderr,"One version scanned with a white sheet of paper behind it\n");
      fprintf(stderr,"and one scanned with a black sheet of paper put behind it instead:\n");
      fprintf(stderr,"The result will be the original image with added transparency information\n\n");
      fprintf(stderr,"Usage:\n\n");
      fprintf(stderr,"%s InputfileScannedWithWhiteBackground.png InputfileScannedWithBlackBackground.png Outputfile.png",argv[0]); 
      exit(1);
    }
  /* load the image */
  printf("Loading the image scanned with a white background\n");
  WhiteBackground = imlib_load_image(argv[1]);
  /* if the load was successful */
  if (WhiteBackground)
    {      
      printf("Loading the image scanned with a black background\n");
      BlackBackgnd=imlib_load_image(argv[2]);
      if(BlackBackgnd)
	{
	  long int width,height,x,y;
	  int i,min_transparency=0,max_transparency=0;
	  long int histogram[256];
	    
	  int min_negative_transparency=0;

	  imlib_context_set_image(WhiteBackground);
	  width=imlib_image_get_width();
	  height=imlib_image_get_height();
	  // Paranoia: What if both images differ in size?
	  imlib_context_set_image(BlackBackgnd);
	  if(width!=imlib_image_get_width()||height!=imlib_image_get_height())
	    {
	      fprintf(stderr,"Images differ in size!\n");
	      exit(-1);
	    }
	  
	  Dest=imlib_create_image(width,height);
	  imlib_context_set_image(Dest);
	  imlib_image_set_has_alpha(1);
	  imlib_image_clear_color(0,0,0,0);

	  {
	    
	    printf("Calculating a transparency histogram of the image\n");	

	    memset(&histogram,0,sizeof(histogram));
	    
	    for(y=0;y<width;y++)
	      for(x=0;x<width;x++)
		{
		  int difference;
		  Imlib_Color Pixel_WhiteBackgnd;
		  Imlib_Color Pixel_BlackBackgnd;
		  imlib_context_set_image(WhiteBackground);
		  imlib_image_query_pixel(x,y,&Pixel_WhiteBackgnd);
		  imlib_context_set_image(BlackBackgnd);
		  imlib_image_query_pixel(x,y,&Pixel_BlackBackgnd);

		  difference=Pixel_WhiteBackgnd.red  -Pixel_BlackBackgnd.red+
		    Pixel_WhiteBackgnd.green-Pixel_BlackBackgnd.green+
		    Pixel_WhiteBackgnd.blue -Pixel_BlackBackgnd.blue;
		  if((difference>=0)&&(difference<256))
		    histogram[difference]++;

		  if(difference<min_negative_transparency)
		    min_negative_transparency=difference;
		}

	    if(min_negative_transparency<-20)
	      printf("Warning: Alignment of the images might be bad: Minimum Transparency=%i\n",
		     min_negative_transparency);

	    printf("Trying to find the transparency span: ");
	    long int peaksteepness_last=0;
	    for(i=10;i<245;i++)
	      {
		int peaksteepness=-histogram[i-1]-histogram[i-2]-histogram[i-3]+
		  histogram[i+1]+histogram[i+2]+histogram[i+3];
		if((peaksteepness<0)&&(peaksteepness_last>0))
		  {
		    if(min_transparency==0) min_transparency=i;
		    max_transparency=i;
		  }
		peaksteepness_last = peaksteepness;
	      }
	    
	    // Allow 10% of noise to be still interpreted as transparent.
	    min_transparency+=(max_transparency-min_transparency)/10;
	    max_transparency-=(max_transparency-min_transparency)/10;
	    
	    printf("Min=%i, Max=%i\n",
		   min_transparency,max_transparency);
	    
	    if (max_transparency<=min_transparency)
	      {
		fprintf(stderr,"Didn't succeed analyzing the transparency histogram.\n");
		exit(-1);
	      }
	    
	    max_transparency = max_transparency-min_transparency;
	    
	    printf("Now calculating the image.\n");
	    for(y=0;y<width;y++)
	      for(x=0;x<width;x++)
		{
		  int difference;
		  Imlib_Color Pixel_WhiteBackgnd;
		  Imlib_Color Pixel_BlackBackgnd;
		  Imlib_Color NewPixel;
		  imlib_context_set_image(BlackBackgnd);
		  imlib_image_query_pixel(x,y,&Pixel_BlackBackgnd);
		  imlib_context_set_image(WhiteBackground);
		  imlib_image_query_pixel(x,y,&Pixel_WhiteBackgnd);

		  // Calculate how big a difference the background makes.
		  difference=Pixel_WhiteBackgnd.red  -Pixel_BlackBackgnd.red+
		    Pixel_WhiteBackgnd.green-Pixel_BlackBackgnd.green+
		    Pixel_WhiteBackgnd.blue -Pixel_BlackBackgnd.blue-min_transparency;

		  
		  if(difference>max_transparency)difference=max_transparency;

		  if(difference<min_transparency)difference=min_transparency;

		  NewPixel.alpha=255*(max_transparency-difference)/(max_transparency-min_transparency);
		  NewPixel.red=  Pixel_WhiteBackgnd.red  *(NewPixel.alpha)/255;
		  NewPixel.green=Pixel_WhiteBackgnd.green*(NewPixel.alpha)/255;
		  NewPixel.blue= Pixel_WhiteBackgnd.blue *(NewPixel.alpha)/255;

		  imlib_context_set_color(NewPixel.red,NewPixel.green,NewPixel.blue,NewPixel.alpha);
		  imlib_context_set_image(Dest);
		  imlib_image_draw_pixel(x,y,1);
		}
	  }
	  imlib_context_set_image(BlackBackgnd);
	  imlib_free_image();
	  imlib_context_set_image(WhiteBackground);
	  imlib_free_image();

	  printf("Saving the image.\n");
	  
	  imlib_context_set_image(Dest);
	  char *tmp;
	  /* argument - i.e. .png = png, .tif = tiff etc. */
	  tmp = strrchr(argv[3], '.');
	  if(tmp)
	    imlib_image_set_format(tmp + 1);
	  /* save the image */
	  imlib_save_image(argv[3]);
	  imlib_free_image();
	}
      else
	{
	  fprintf(stderr,"Error loading image %s!\n",argv[1]);
	  exit(-1);
	}
    }
  else
    {
      fprintf(stderr,"Error loading image %s!\n",argv[2]);
      exit(-1);
    }
  exit(0);
}
